@@ -32,15 +32,17 @@ LauncherItem::LauncherItem(const QString &appId, const QString &name, const QStr
     m_recent(false),
     m_progress(-1),
     m_count(0),
+    m_countVisible(false),
     m_focused(false),
     m_quickList(new QuickListModel(this))
 {
     QuickListEntry nameAction;
+    nameAction.setActionId("launch_item");
     nameAction.setText(m_name);
     m_quickList->appendAction(nameAction);
     QuickListEntry pinningAction;
     pinningAction.setActionId("pin_item");
-    pinningAction.setText(gettext("Lock to Launcher"));
+    pinningAction.setText(gettext("Pin shortcut"));
     m_quickList->appendAction(pinningAction);
 }
 
@@ -54,11 +56,27 @@ QString LauncherItem::name() const
     return m_name;
 }
 
+void LauncherItem::setName(const QString &name)
+{
+    if (m_name != name) {
+        m_name = name;
+        Q_EMIT nameChanged(name);
+    }
+}
+
 QString LauncherItem::icon() const
 {
     return m_icon;
 }
 
+void LauncherItem::setIcon(const QString &icon)
+{
+    if (m_icon != icon) {
+        m_icon = icon;
+        Q_EMIT iconChanged(icon);
+    }
+}
+
 bool LauncherItem::pinned() const
 {
     return m_pinned;
@@ -70,7 +88,7 @@ void LauncherItem::setPinned(bool pinned)
         m_pinned = pinned;
         QuickListEntry entry;
         entry.setActionId("pin_item");
-        entry.setText(pinned ? gettext("Unlock from Launcher") : gettext("Lock to Launcher"));
+        entry.setText(pinned ? gettext("Unpin shortcut") : gettext("Pin shortcut"));
         m_quickList->updateAction(entry);
         Q_EMIT pinnedChanged(pinned);
     }
@@ -128,6 +146,19 @@ void LauncherItem::setCount(int count)
     }
 }
 
+bool LauncherItem::countVisible() const
+{
+    return m_countVisible;
+}
+
+void LauncherItem::setCountVisible(bool countVisible)
+{
+    if (m_countVisible != countVisible) {
+        m_countVisible = countVisible;
+        Q_EMIT countVisibleChanged(countVisible);
+    }
+}
+
 bool LauncherItem::focused() const
 {
     return m_focused;
