@@ -26,16 +26,14 @@
 #include "plugin.h"
 
 // local
-#include "bottombarvisibilitycommunicatorshell.h"
 #include "qlimitproxymodelqml.h"
 #include "qsortfilterproxymodelqml.h"
+#include "relativetimeformatter.h"
 #include "timeformatter.h"
 #include "unitymenumodelpaths.h"
+#include "windowkeysfilter.h"
 #include "easingcurve.h"
 
-static const char* BOTTOM_BAR_VISIBILITY_COMMUNICATOR_DBUS_PATH = "/BottomBarVisibilityCommunicator";
-static const char* DBUS_SERVICE = "com.canonical.Shell.BottomBarVisibilityCommunicator";
-
 void UtilsPlugin::registerTypes(const char *uri)
 {
     Q_ASSERT(uri == QLatin1String("Utils"));
@@ -44,17 +42,13 @@ void UtilsPlugin::registerTypes(const char *uri)
     qmlRegisterType<QSortFilterProxyModelQML>(uri, 0, 1, "SortFilterProxyModel");
     qmlRegisterType<UnityMenuModelPaths>(uri, 0, 1, "UnityMenuModelPaths");
     qmlRegisterType<TimeFormatter>(uri, 0, 1, "TimeFormatter");
+    qmlRegisterType<WindowKeysFilter>(uri, 0, 1, "WindowKeysFilter");
     qmlRegisterType<GDateTimeFormatter>(uri, 0, 1, "GDateTimeFormatter");
-    qmlRegisterUncreatableType<BottomBarVisibilityCommunicatorShell>(uri, 0, 1, "BottomBarVisibilityCommunicatorShell", "Can't create BottomBarVisibilityCommunicatorShell");
     qmlRegisterType<EasingCurve>(uri, 0, 1, "EasingCurve");
+    qmlRegisterType<RelativeTimeFormatter>(uri, 0, 1, "RelativeTimeFormatter");
 }
 
 void UtilsPlugin::initializeEngine(QQmlEngine *engine, const char *uri)
 {
     QQmlExtensionPlugin::initializeEngine(engine, uri);
-
-    QDBusConnection::sessionBus().registerService(DBUS_SERVICE);
-    BottomBarVisibilityCommunicatorShell *bottomBarVisibilityCommunicator = &BottomBarVisibilityCommunicatorShell::instance();
-    QDBusConnection::sessionBus().registerObject(BOTTOM_BAR_VISIBILITY_COMMUNICATOR_DBUS_PATH, bottomBarVisibilityCommunicator, QDBusConnection::ExportAllContents);
-    engine->rootContext()->setContextProperty("bottomBarVisibilityCommunicatorShell", bottomBarVisibilityCommunicator);
 }
