@@ -97,7 +97,8 @@ class ApplicationLifecycleTests(tests.ApplicationLifeCycleTestCase):
         self.assertThat(greeter.created, Eventually(Equals(True)))
 
         application_name = self.launch_fake_app()
-        self.assertThat(greeter.created, Eventually(Equals(False)))
+        greeter.wait_swiped_away()
+        process_helpers.unlock_unity()
         self.assert_current_focused_application(application_name)
 
     def test_greeter_hides_on_app_focus(self):
@@ -106,12 +107,28 @@ class ApplicationLifecycleTests(tests.ApplicationLifeCycleTestCase):
         self.assert_current_focused_application(application_name)
 
         self.main_window.show_dash_swiping()
-        self.assert_current_focused_application('')
+        self.assert_current_focused_application('unity8-dash')
 
         process_helpers.lock_unity()
         greeter = self.main_window.get_greeter()
         self.assertThat(greeter.created, Eventually(Equals(True)))
 
         self.launch_upstart_application(application_name)
-        self.assertThat(greeter.created, Eventually(Equals(False)))
+        greeter.wait_swiped_away()
+        process_helpers.unlock_unity()
+        self.assert_current_focused_application(application_name)
+
+    def test_click_dash_icon_must_unfocus_application(self):
+        application_name = self.launch_fake_app()
+        self.assert_current_focused_application(application_name)
+
+        self.main_window.show_dash_from_launcher()
+
+        self.assert_current_focused_application('unity8-dash')
+
+    def test_click_app_icon_on_dash_must_focus_it(self):
+        application_name = self.launch_fake_app()
+        self.main_window.show_dash_from_launcher()
+
+        self.main_window.launch_application(application_name)
         self.assert_current_focused_application(application_name)
