@@ -17,15 +17,21 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+import ubuntuuitoolkit
+
 from unity8.shell.emulators import UnityEmulatorBase
 
 
 class Greeter(UnityEmulatorBase):
     """An emulator that understands the greeter screen."""
 
+    def wait_swiped_away(self):
+        self.showProgress.wait_for(0)
+
     def swipe(self):
         """Swipe the greeter screen away."""
         self.created.wait_for(True)
+        self.showProgress.wait_for(1)
 
         rect = self.globalRect
         start_x = rect[0] + rect[2] - 3
@@ -34,7 +40,8 @@ class Greeter(UnityEmulatorBase):
         stop_y = start_y
         self.pointing_device.drag(start_x, start_y, stop_x, stop_y)
 
-        self.created.wait_for(False)
+        self.wait_swiped_away()
 
     def get_prompt(self):
-        return self.select_single("TextField", objectName="passwordInput")
+        return self.select_single(
+            ubuntuuitoolkit.TextField, objectName='passwordInput')
