@@ -1,7 +1,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Unity Autopilot Test Suite
-# Copyright (C) 2012-2013 Canonical
+# Copyright (C) 2012, 2013, 2014 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,14 +20,11 @@
 
 from __future__ import absolute_import
 
-from unity8.process_helpers import unlock_unity
 from unity8.shell import with_lightdm_mock
 from unity8.shell.tests import UnityTestCase, _get_device_emulation_scenarios
 
 from autopilot.matchers import Eventually
-from autopilot.platform import model
 import sys
-from testtools import skipUnless
 from testtools.matchers import Equals
 import logging
 
@@ -47,26 +44,26 @@ class TestLockscreen(UnityTestCase):
     @with_lightdm_mock("single-pin")
     def test_can_unlock_pin_screen(self):
         """Must be able to unlock the PIN entry lock screen."""
-        unity_proxy = self.launch_unity()
+        self.launch_unity()
         greeter = self.main_window.get_greeter()
 
         if greeter.narrowMode:
-            unlock_unity(unity_proxy)
+            greeter.swipe()
             lockscreen = self._wait_for_lockscreen()
-            self._enter_pincode("1234")
+            self.main_window.enter_pin_code("1234")
             self.assertThat(lockscreen.shown, Eventually(Equals(False)))
         else:
-            self._enter_prompt_passphrase("1234")
+            self._enter_prompt_passphrase("1234\n")
             self.assertThat(greeter.shown, Eventually(Equals(False)))
 
     @with_lightdm_mock("single-passphrase")
     def test_can_unlock_passphrase_screen(self):
         """Must be able to unlock the passphrase entry screen."""
-        unity_proxy = self.launch_unity()
+        self.launch_unity()
         greeter = self.main_window.get_greeter()
 
         if greeter.narrowMode:
-            unlock_unity(unity_proxy)
+            greeter.swipe()
             lockscreen = self._wait_for_lockscreen()
             self._enter_pin_passphrase("password")
             self.assertThat(lockscreen.shown, Eventually(Equals(False)))
@@ -77,18 +74,18 @@ class TestLockscreen(UnityTestCase):
     @with_lightdm_mock("single-pin")
     def test_pin_screen_wrong_code(self):
         """Entering the wrong pin code must not dismiss the lock screen."""
-        unity_proxy = self.launch_unity()
+        self.launch_unity()
         greeter = self.main_window.get_greeter()
 
         if greeter.narrowMode:
-            unlock_unity(unity_proxy)
+            greeter.swipe()
             lockscreen = self._wait_for_lockscreen()
-            self._enter_pincode("4321")
+            self.main_window.enter_pin_code("4321")
             pinentryField = self.main_window.get_pinentryField()
             self.assertThat(pinentryField.text, Eventually(Equals("")))
             self.assertThat(lockscreen.shown, Eventually(Equals(True)))
         else:
-            self._enter_prompt_passphrase("4231")
+            self._enter_prompt_passphrase("4231\n")
             prompt = self.main_window.get_greeter().get_prompt()
             self.assertThat(prompt.text, Eventually(Equals("")))
             self.assertThat(greeter.shown, Eventually(Equals(True)))
@@ -96,11 +93,11 @@ class TestLockscreen(UnityTestCase):
     @with_lightdm_mock("single-passphrase")
     def test_passphrase_screen_wrong_password(self):
         """Entering the wrong password must not dismiss the lock screen."""
-        unity_proxy = self.launch_unity()
+        self.launch_unity()
         greeter = self.main_window.get_greeter()
 
         if greeter.narrowMode:
-            unlock_unity(unity_proxy)
+            greeter.swipe()
             lockscreen = self._wait_for_lockscreen()
             self._enter_pin_passphrase("foobar")
             pinentryField = self.main_window.get_pinentryField()
@@ -120,28 +117,6 @@ class TestLockscreen(UnityTestCase):
         self.assertThat(lockscreen.shown, Eventually(Equals(True)))
         return lockscreen
 
-    def _enter_pincode(self, code):
-        """Enter code 'code' into the single-pin lightdm pincode entry
-        screen.
-
-        :param code: must be a string of numeric characters.
-        :raises: TypeError if code is not a string.
-        :raises: ValueError if code contains non-numeric characters.
-
-        """
-
-        if not isinstance(code, basestring):
-            raise TypeError(
-                "'code' parameter must be a string, not %r."
-                % type(passphrase)
-            )
-        for num in code:
-            if not num.isdigit():
-                raise ValueError(
-                    "'code' parameter contains non-numeric characters."
-                )
-            self.touch.tap_object(self.main_window.get_pinPadButton(int(num)))
-
     def _enter_pin_passphrase(self, passphrase):
         """Enter the password specified in 'passphrase' into the password entry
         field of the pin lock screen.
@@ -150,18 +125,17 @@ class TestLockscreen(UnityTestCase):
         :raises: TypeError if passphrase is not a string.
 
         """
-        if not isinstance(passphrase, basestring):
+        if not isinstance(passphrase, str):
             raise TypeError(
                 "'passphrase' parameter must be a string, not %r."
                 % type(passphrase)
             )
 
-        pinentryField = self.main_window.get_pinentryField()
-        self.touch.tap_object(pinentryField)
-        self.assertThat(pinentryField.activeFocus, Eventually(Equals(True)))
-        for character in passphrase:
-            self._type_character(character, pinentryField)
-        logger.debug("Typed passphrase: %s", pinentryField.text)
+        pin_entry_field = self.main_window.get_pinentryField()
+        # pinentryField should automatically have focus
+        self.keyboard.type(passphrase)
+        logger.debug("Typed passphrase: %s", pin_entry_field.text)
+        self.assertEqual(pin_entry_field.text, passphrase)
         self.keyboard.type("\n")
 
     def _enter_prompt_passphrase(self, passphrase):
@@ -179,20 +153,6 @@ class TestLockscreen(UnityTestCase):
             )
 
         prompt = self.main_window.get_greeter().get_prompt()
-        self.touch.tap_object(prompt)
-        self.assertThat(prompt.activeFocus, Eventually(Equals(True)))
-        for character in passphrase:
-            self._type_character(character, prompt)
+        prompt.write(passphrase)
         logger.debug("Typed passphrase: %s", prompt.text)
         self.keyboard.type("\n")
-
-    def _type_character(self, character, prompt, retries=5):
-        current_text = prompt.text
-        self.keyboard.type(character)
-        try:
-            self.assertThat(prompt.text, Eventually(Equals(current_text + character)))
-        except AssertionError:
-            if retries > 0:
-                self._type_character(character, prompt, retries-1)
-            else:
-                raise
