@@ -21,6 +21,7 @@
 
 from __future__ import absolute_import
 
+from unity8 import shell
 from unity8.process_helpers import unlock_unity
 from unity8.shell.tests import UnityTestCase, _get_device_emulation_scenarios
 
@@ -64,7 +65,8 @@ class NotificationsBase(UnityTestCase):
         if os.path.abspath(__file__).startswith('/usr/'):
             return '/usr/share/unity8/graphics/' + icon_name
         else:
-            return os.path.dirname(__file__) + "/../../../../../qml/graphics/" + icon_name
+            return os.path.dirname(__file__) + (
+                "/../../../../../qml/graphics/" + icon_name)
 
     def _get_notifications_list(self):
         return self.main_window.select_single(
@@ -133,10 +135,7 @@ class InteractiveNotificationBase(NotificationsBase):
         actions = [("action_id", "dummy")]
         hints = [
             ("x-canonical-switch-to-application", "true"),
-            (
-                "x-canonical-secondary-icon",
-                self._get_icon_path('applicationIcons/phone-app.png')
-            )
+            ("x-canonical-secondary-icon","dialer")
         ]
 
         self._create_interactive_notification(
@@ -152,7 +151,7 @@ class InteractiveNotificationBase(NotificationsBase):
             'Notification', objectName='notification1')
         notification = get_notification()
 
-        self.touch.tap_object(
+        notification.pointing_device.click_object(
             notification.select_single(objectName="interactiveArea")
         )
 
@@ -168,19 +167,20 @@ class InteractiveNotificationBase(NotificationsBase):
         body = "Frank Zappa\n+44 (0)7736 027340"
         icon_path = self._get_icon_path('avatars/anna_olsson.png')
         hints = [
-            (
-                "x-canonical-secondary-icon",
-                self._get_icon_path('applicationIcons/phone-app.png')
-            ),
+            ("x-canonical-secondary-icon", "incoming-call"),
             ("x-canonical-snap-decisions", "true"),
+            ("x-canonical-private-affirmative-tint", "true"),
+            ("x-canonical-private-rejection-tint", "true"),
         ]
 
         actions = [
-            ('action_accept', 'Accept'),
-            ('action_decline_1', 'Decline'),
-            ('action_decline_2', '"Can\'t talk now, what\'s up?"'),
-            ('action_decline_3', '"I call you back."'),
-            ('action_decline_4', 'Send custom message...'),
+            ('action_accept', 'Hold + Answer'),
+            ('action_decline_1', 'End + Answer'),
+            ('action_decline_2', 'Decline'),
+            ('action_decline_3', 'message:I missed your call - can you call me now?'),
+            ('action_decline_4', 'message:I\'m running late. I\'m on my way.'),
+            ('action_decline_5', 'message:I\'m busy at the moment. I\'ll call later.'),
+            ('action_decline_6', 'edit:Custom'),
         ]
 
         self._create_interactive_notification(
@@ -197,18 +197,148 @@ class InteractiveNotificationBase(NotificationsBase):
             'Notification', objectName='notification1')
         notification = get_notification()
         self._assert_notification(notification, summary, body, True, True, 1.0)
-        initial_height = notification.height
-        self.touch.tap_object(notification.select_single(objectName="button1"))
+        notification.pointing_device.click_object(
+            notification.select_single(objectName="combobutton_dropdown"))
         self.assertThat(
-            notification.height,
-            Eventually(Equals(initial_height +
-                              3 * notification.select_single(
-                                  objectName="buttonColumn").spacing +
-                              3 * notification.select_single(
-                                  objectName="button4").height)))
-        self.touch.tap_object(notification.select_single(objectName="button4"))
+            notification.select_single(objectName="notify_button2").expanded,
+            Eventually(Equals(True)))
+        time.sleep(2)
+        notification.pointing_device.click_object(
+            notification.select_single(objectName="notify_button4"))
         self.assert_notification_action_id_was_called("action_decline_4")
 
+
+    def test_sd_one_over_two_layout(self):
+        """Snap-decision with three actions should use one-over two button layout."""
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
+
+        summary = "Theatre at Ferria Stadium"
+        body = "at Ferria Stadium in Bilbao, Spain\n07578545317"
+        hints = [
+            ("x-canonical-snap-decisions", "true"),
+            ("x-canonical-non-shaped-icon", "true"),
+            ("x-canonical-private-affirmative-tint", "true")
+        ]
+
+        actions = [
+            ('action_accept', 'Ok'),
+            ('action_decline_1', 'Snooze'),
+            ('action_decline_2', 'View'),
+        ]
+
+        self._create_interactive_notification(
+            summary,
+            body,
+            None,
+            "NORMAL",
+            actions,
+            hints
+        )
+
+        # verify and interact with the triggered snap-decision notification
+        notify_list = self._get_notifications_list()
+        get_notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
+        notification = get_notification()
+        self._assert_notification(
+            notification, summary, body, False, False, 1.0)
+        notification.pointing_device.click_object(
+            notification.select_single(objectName="notify_oot_button0"))
+        self.assert_notification_action_id_was_called("action_accept")
+
+    def test_modal_sd_without_greeter(self):
+        """Snap-decision should block input to shell without greeter."""
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
+
+        summary = "Incoming file"
+        body = "Frank would like to send you the file: essay.pdf"
+        icon_path = "sync-idle"
+        hints = [
+            ("x-canonical-snap-decisions", "true"),
+            ("x-canonical-non-shaped-icon", "true"),
+            ("x-canonical-private-affirmative-tint", "true"),
+            ("x-canonical-private-rejection-tint", "true"),
+        ]
+
+        actions = [
+            ('action_accept', 'Accept'),
+            ('action_decline_1', 'Decline'),
+        ]
+
+        self._create_interactive_notification(
+            summary,
+            body,
+            icon_path,
+            "NORMAL",
+            actions,
+            hints
+        )
+
+        # verify that we cannot reveal the launcher (no longer interact with
+        # the shell)
+        time.sleep(1)
+        self.main_window.show_dash_swiping()
+        self.assertThat(
+            self.main_window.is_launcher_open, Eventually(Equals(False)))
+
+        # verify and interact with the triggered snap-decision notification
+        notify_list = self._get_notifications_list()
+        get_notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
+        notification = get_notification()
+        self._assert_notification(
+            notification, summary, body, True, False, 1.0)
+        notification.pointing_device.click_object(
+            notification.select_single(objectName="notify_button0"))
+        self.assert_notification_action_id_was_called("action_accept")
+
+    def test_modal_sd_with_greeter(self):
+        """A snap-decision should not block input to the greeter beneath it."""
+        self.launch_unity()
+
+        summary = "Incoming file"
+        body = "Frank would like to send you the file: essay.pdf"
+        icon_path = "sync-idle"
+        hints = [
+            ("x-canonical-snap-decisions", "true"),
+            ("x-canonical-non-shaped-icon", "true"),
+            ("x-canonical-private-affirmative-tint", "true"),
+            ("x-canonical-private-rejection-tint", "true"),
+        ]
+
+        actions = [
+            ('action_accept', 'Accept'),
+            ('action_decline_1', 'Decline'),
+        ]
+
+        self._create_interactive_notification(
+            summary,
+            body,
+            icon_path,
+            "NORMAL",
+            actions,
+            hints
+        )
+
+        # verify that we can swipe away the greeter (interact with the "shell")
+        time.sleep(1)
+        self.main_window.show_dash_swiping()
+        greeter = self.main_window.get_greeter()
+        self.assertThat(greeter.shown, Eventually(Equals(False)))
+
+        # verify and interact with the triggered snap-decision notification
+        notify_list = self._get_notifications_list()
+        get_notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
+        notification = get_notification()
+        self._assert_notification(
+            notification, summary, body, True, False, 1.0)
+        notification.pointing_device.click_object(
+            notification.select_single(objectName="notify_button0"))
+        self.assert_notification_action_id_was_called("action_accept")
+
     def _create_interactive_notification(
         self,
         summary="",
@@ -348,13 +478,10 @@ class EphemeralNotificationsTests(NotificationsBase):
                "join me and Anna?"
         icon_path = self._get_icon_path('avatars/anna_olsson.png')
         hints = [
-            (
-                "x-canonical-secondary-icon",
-                self._get_icon_path('applicationIcons/phone-app.png')
-            )
+            ("x-canonical-secondary-icon", "message")
         ]
 
-        notification = self._create_ephemeral_notification(
+        notification = shell.create_ephemeral_notification(
             summary,
             body,
             icon_path,
@@ -384,17 +511,13 @@ class EphemeralNotificationsTests(NotificationsBase):
         notify_list = self._get_notifications_list()
 
         summary = "Upload of image completed"
-        hints = [
-            (
-                "x-canonical-secondary-icon",
-                self._get_icon_path('applicationIcons/facebook.png')
-            )
-        ]
+        icon_path = self._get_icon_path('applicationIcons/facebook.png')
+        hints=[]
 
-        notification = self._create_ephemeral_notification(
+        notification = shell.create_ephemeral_notification(
             summary,
             None,
-            None,
+            icon_path,
             hints,
             "NORMAL",
         )
@@ -407,8 +530,8 @@ class EphemeralNotificationsTests(NotificationsBase):
             notification(),
             summary,
             None,
-            False,
             True,
+            False,
             1.0
         )
 
@@ -433,7 +556,7 @@ class EphemeralNotificationsTests(NotificationsBase):
         body_critical = 'Dude, this is so urgent you have no idea :)'
         icon_path_critical = self._get_icon_path('avatars/anna_olsson.png')
 
-        notification_normal = self._create_ephemeral_notification(
+        notification_normal = shell.create_ephemeral_notification(
             summary_normal,
             body_normal,
             icon_path_normal,
@@ -441,7 +564,7 @@ class EphemeralNotificationsTests(NotificationsBase):
         )
         notification_normal.show()
 
-        notification_low = self._create_ephemeral_notification(
+        notification_low = shell.create_ephemeral_notification(
             summary_low,
             body_low,
             icon_path_low,
@@ -449,7 +572,7 @@ class EphemeralNotificationsTests(NotificationsBase):
         )
         notification_low.show()
 
-        notification_critical = self._create_ephemeral_notification(
+        notification_critical = shell.create_ephemeral_notification(
             summary_critical,
             body_critical,
             icon_path_critical,
@@ -510,7 +633,7 @@ class EphemeralNotificationsTests(NotificationsBase):
         summary = 'Summary-Body'
         body = 'This is a superfluous notification'
 
-        notification = self._create_ephemeral_notification(summary, body)
+        notification = shell.create_ephemeral_notification(summary, body)
         notification.show()
 
         notification = notify_list.wait_select_single(
@@ -534,7 +657,7 @@ class EphemeralNotificationsTests(NotificationsBase):
 
         summary = 'Summary-Only'
 
-        notification = self._create_ephemeral_notification(summary)
+        notification = shell.create_ephemeral_notification(summary)
         notification.show()
 
         notification = notify_list.wait_select_single(
@@ -554,7 +677,7 @@ class EphemeralNotificationsTests(NotificationsBase):
         body = 'This is the original content of this notification-bubble.'
         icon_path = self._get_icon_path('avatars/funky.png')
 
-        notification = self._create_ephemeral_notification(
+        notification = shell.create_ephemeral_notification(
             summary,
             body,
             icon_path
@@ -578,7 +701,8 @@ class EphemeralNotificationsTests(NotificationsBase):
         icon_path = self._get_icon_path('avatars/amanda.png')
         notification.update(summary, body, icon_path)
         notification.show()
-        self._assert_notification(get_notification(), summary, body, True, False, 1.0)
+        self._assert_notification(
+            get_notification(), summary, body, True, False, 1.0)
 
     def test_update_notification_layout_change(self):
         """Notification must allow updating its contents and layout while
@@ -592,9 +716,9 @@ class EphemeralNotificationsTests(NotificationsBase):
         body = 'This bubble uses the icon-title-body layout with a ' \
             'secondary icon.'
         icon_path = self._get_icon_path('avatars/anna_olsson.png')
-        hint_icon = self._get_icon_path('applicationIcons/phone-app.png')
+        hint_icon = 'dialer'
 
-        notification = self._create_ephemeral_notification(
+        notification = shell.create_ephemeral_notification(
             summary,
             body,
             icon_path
@@ -625,118 +749,5 @@ class EphemeralNotificationsTests(NotificationsBase):
         notification.show()
 
         self.assertThat(get_notification, Eventually(NotEquals(None)))
-        self._assert_notification(get_notification(), summary, body, False, False, 1.0)
-
-    def test_append_hint(self):
-        """Notification has to accumulate body-text using append-hint."""
-        unity_proxy = self.launch_unity()
-        unlock_unity(unity_proxy)
-
-        notify_list = self._get_notifications_list()
-
-        summary = 'Cole Raby'
-        body = 'Hey Bro Coly!'
-        icon_path = self._get_icon_path('avatars/amanda.png')
-        body_sum = body
-        notification = self._create_ephemeral_notification(
-            summary,
-            body,
-            icon_path,
-            hints=[('x-canonical-append', 'true')]
-        )
-
-        notification.show()
-
-        get_notification = lambda: notify_list.wait_select_single(
-            'Notification', objectName='notification1')
-
-        notification = get_notification()
         self._assert_notification(
-            notification,
-            summary,
-            body_sum,
-            True,
-            False,
-            1.0
-        )
-
-        bodies = [
-            'What\'s up dude?',
-            'Did you watch the air-race in Oshkosh last week?',
-            'Phil owned the place like no one before him!',
-            'Did really everything in the race work according to regulations?',
-            'Somehow I think to remember Burt Williams did cut corners and '
-            'was not punished for this.',
-            'Hopefully the referees will watch the videos of the race.',
-            'Burt could get fined with US$ 50000 for that rule-violation :)'
-        ]
-
-        for new_body in bodies:
-            body = new_body
-            body_sum += '\n' + body
-            notification = self._create_ephemeral_notification(
-                summary,
-                body,
-                icon_path,
-                hints=[('x-canonical-append', 'true')]
-            )
-            notification.show()
-
-            get_notification = lambda: notify_list.wait_select_single(
-                'Notification',
-                objectName='notification1'
-            )
-            notification = get_notification()
-            self._assert_notification(
-                notification,
-                summary,
-                body_sum,
-                True,
-                False,
-                1.0
-            )
-
-    def _create_ephemeral_notification(
-        self,
-        summary="",
-        body="",
-        icon=None,
-        hints=[],
-        urgency="NORMAL"
-    ):
-        """Create an ephemeral (non-interactive) notification
-
-            :param summary: Summary text for the notification
-            :param body: Body text to display in the notification
-            :param icon: Path string to the icon to use
-            :param hint_strings: List of tuples containing the 'name' and value
-                for setting the hint strings for the notification
-            :param urgency: Urgency string for the noticiation, either: 'LOW',
-                'NORMAL', 'CRITICAL'
-
-        """
-        logger.info(
-            "Creating ephemeral: summary(%s), body(%s), urgency(%r) "
-            "and Icon(%s)",
-            summary,
-            body,
-            urgency,
-            icon
-        )
-
-        n = Notify.Notification.new(summary, body, icon)
-
-        for hint in hints:
-            key, value = hint
-            n.set_hint_string(key, value)
-            logger.info("Adding hint to notification: (%s, %s)", key, value)
-        n.set_urgency(self._get_urgency(urgency))
-
-        return n
-
-    def _get_urgency(self, urgency):
-        """Translates urgency string to enum."""
-        _urgency_enums = {'LOW': Notify.Urgency.LOW,
-                          'NORMAL': Notify.Urgency.NORMAL,
-                          'CRITICAL': Notify.Urgency.CRITICAL}
-        return _urgency_enums.get(urgency.upper())
+            get_notification(), summary, body, False, False, 1.0)
