@@ -19,8 +19,6 @@
 #include "../Greeter.h"
 #include "../GreeterPrivate.h"
 
-#include <QDebug>
-
 namespace QLightDM
 {
 
@@ -34,20 +32,15 @@ GreeterPrivate::GreeterPrivate(Greeter* parent)
 void GreeterPrivate::handleAuthenticate()
 {
     Q_Q(Greeter);
-
-    qDebug() << "handleAuthentication called!" << authenticationUser;
-
-    Q_EMIT q->showPrompt("Password:", Greeter::PromptTypeSecret);
+    Q_EMIT q->showPrompt("Password: ", Greeter::PromptTypeSecret);
 }
 
 void GreeterPrivate::handleRespond(const QString &response)
 {
     Q_Q(Greeter);
 
-
     authenticated = (response == "password");
-    qDebug() << "responding" << response << authenticated;
-    Q_EMIT q->authenticationComplete();
+    q->sendAuthenticationComplete();
 }
 
 }
