@@ -18,39 +18,28 @@
 #ifndef FAKE_PREVIEWSTACK_H
 #define FAKE_PREVIEWSTACK_H
 
-#include <QAbstractListModel>
+#include <unity/shell/scopes/PreviewStackInterface.h>
+
 #include <QSharedPointer>
 #include <QVariantMap>
 
 class PreviewModel;
 
-class PreviewStack : public QAbstractListModel
+class PreviewStack : public unity::shell::scopes::PreviewStackInterface
 {
     Q_OBJECT
 
-    Q_ENUMS(Roles)
-
-    Q_PROPERTY(int widgetColumnCount READ widgetColumnCount WRITE setWidgetColumnCount NOTIFY widgetColumnCountChanged)
-
 public:
     explicit PreviewStack(QObject* parent = 0);
     virtual ~PreviewStack();
 
-    enum Roles {
-        RolePreviewModel
-    };
-
     QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
-    QHash<int, QByteArray> roleNames() const override;
     int rowCount(const QModelIndex& parent = QModelIndex()) const override;
 
-    Q_INVOKABLE PreviewModel* get(int index) const;
+    Q_INVOKABLE unity::shell::scopes::PreviewModelInterface* getPreviewModel(int index) const override;
 
-    void setWidgetColumnCount(int columnCount);
-    int widgetColumnCount() const;
-
-Q_SIGNALS:
-    void widgetColumnCountChanged();
+    void setWidgetColumnCount(int columnCount) override;
+    int widgetColumnCount() const override;
 
 private:
     QList<PreviewModel*> m_previews;
