@@ -18,6 +18,7 @@
 
 // Self
 #include "fake_scopes.h"
+#include "fake_scopesoverview.h"
 
 // TODO: Implement remaining pieces, like Categories (i.e. LensView now gives warnings)
 
@@ -25,22 +26,19 @@
 #include <QTimer>
 
 Scopes::Scopes(QObject *parent)
-: QAbstractListModel(parent)
-, m_loaded(false)
-, timer(this)
+ : unity::shell::scopes::ScopesInterface(parent)
+ , m_scopesOverview(nullptr)
+ , m_loaded(false)
+ , timer(this)
 {
-    m_roles[Scopes::RoleScope] = "scope";
-    m_roles[Scopes::RoleId] = "id";
-    m_roles[Scopes::RoleVisible] = "visible";
-    m_roles[Scopes::RoleTitle] = "title";
+    timer.setSingleShot(true);
+    timer.setInterval(100);
+    QObject::connect(&timer, SIGNAL(timeout()), this, SLOT(updateScopes()));
 
     QObject::connect(this, SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SIGNAL(countChanged()));
     QObject::connect(this, SIGNAL(rowsRemoved(const QModelIndex &, int, int)), this, SIGNAL(countChanged()));
     QObject::connect(this, SIGNAL(modelReset()), this, SIGNAL(countChanged()));
 
-    timer.setSingleShot(true);
-    timer.setInterval(100);
-    QObject::connect(&timer, SIGNAL(timeout()), this, SLOT(updateScopes()));
     load();
 }
 
@@ -55,10 +53,19 @@ void Scopes::updateScopes()
     addScope(new Scope("MockScope2", "Music", false, this));
     addScope(new Scope("clickscope", "Apps", true, this));
     addScope(new Scope("MockScope5", "Videos", true, this));
+    addScope(new Scope("SingleCategoryScope", "Single", true, this, 1));
+    addScope(new Scope("MockScope4", "MS4", true, this));
+    addScope(new Scope("MockScope6", "MS6", true, this));
+    addScope(new Scope("MockScope7", "MS7", false, this));
+    addScope(new Scope("MockScope8", "MS8", false, this));
+    addScope(new Scope("MockScope9", "MS9", false, this));
+    addScope(new Scope("NullPreviewScope", "NPS", false, this, 1, true));
+    m_scopesOverview = new ScopesOverview(this);
 
     if (!m_loaded) {
         m_loaded = true;
-        Q_EMIT loadedChanged(m_loaded);
+        Q_EMIT loadedChanged();
+        Q_EMIT overviewScopeChanged();
     }
 }
 
@@ -67,14 +74,17 @@ void Scopes::clear()
     timer.stop();
     if (m_scopes.size() > 0) {
         beginRemoveRows(QModelIndex(), 0, m_scopes.count()-1);
-        qDeleteAll(m_scopes);
+        qDeleteAll(m_allScopes);
+        m_allScopes.clear();
         m_scopes.clear();
         endRemoveRows();
     }
+    delete m_scopesOverview;
+    m_scopesOverview = nullptr;
 
     if (m_loaded) {
         m_loaded = false;
-        Q_EMIT loadedChanged(m_loaded);
+        Q_EMIT loadedChanged();
     }
 }
 
@@ -83,11 +93,6 @@ void Scopes::load()
     timer.start();
 }
 
-QHash<int, QByteArray> Scopes::roleNames() const
-{
-    return m_roles;
-}
-
 int Scopes::rowCount(const QModelIndex&) const
 {
     return m_scopes.count();
@@ -105,8 +110,6 @@ QVariant Scopes::data(const QModelIndex& index, int role) const
         return QVariant::fromValue(scope);
     } else if (role == Scopes::RoleId) {
         return QVariant::fromValue(scope->id());
-    } else if (role == Scopes::RoleVisible) {
-        return QVariant::fromValue(scope->visible());
     } else if (role == Scopes::RoleTitle) {
         return QVariant::fromValue(scope->name());
     } else {
@@ -114,14 +117,32 @@ QVariant Scopes::data(const QModelIndex& index, int role) const
     }
 }
 
-QVariant Scopes::get(int row) const
+unity::shell::scopes::ScopeInterface* Scopes::getScope(int row) const
 {
-    return data(QAbstractListModel::index(row), 0);
+    if (row < 0 || row >= m_scopes.size()) {
+        return nullptr;
+    }
+
+    return m_scopes[row];
 }
 
-QVariant Scopes::get(QString const&) const
+unity::shell::scopes::ScopeInterface* Scopes::getScope(QString const &scope_id) const
 {
-    return QVariant();
+    // According to mh3 Scopes::getScope should only return favorite scopes (i.e the ones in the model)
+    for (Scope *scope : m_scopes) {
+        if (scope->id() == scope_id)
+            return scope;
+    }
+    return nullptr;
+}
+
+Scope* Scopes::getScopeFromAll(const QString& scope_id) const
+{
+    for (Scope *scope : m_allScopes) {
+        if (scope->id() == scope_id)
+            return scope;
+    }
+    return nullptr;
 }
 
 QModelIndex Scopes::parent(const QModelIndex&) const
@@ -139,10 +160,28 @@ int Scopes::count() const
     return rowCount();
 }
 
+unity::shell::scopes::ScopeInterface* Scopes::overviewScope() const
+{
+    return m_scopesOverview;
+}
+
+QList<Scope*> Scopes::scopes() const
+{
+    return m_scopes;
+}
+
+QList<Scope*> Scopes::allScopes() const
+{
+    return m_allScopes;
+}
+
 void Scopes::addScope(Scope* scope)
 {
     int index = rowCount();
-    beginInsertRows(QModelIndex(), index, index);
-    m_scopes.append(scope);
-    endInsertRows();
+    if (scope->favorite()) {
+        beginInsertRows(QModelIndex(), index, index);
+        m_scopes.append(scope);
+        endInsertRows();
+    }
+    m_allScopes.append(scope);
 }
